/*
  GPS_Chiming__Multi_Clock_N5IB_v3r00.ino
  Matches hardware schematic and PC boards v2.00 and 2.01
  22 January 2020 

  Add barometric sensor on I2C bus
  
  Uses Software Serial to implement multple serial ports:
  
  Receiving from NMEA data from GPS, selecting a given NMEA sentence
  and parsing the sentence to extract relevant data,
  echo selected NMEA sentences to the serial monitor,
  and display parsed data to an LCD panel

  Commuicates with MP3 player via a second software serial port

  Uses I2C protocol to control an LCD display
  
 modified 25 May 2012, by Tom Igoe, based on Mikal Hart's example
 Modified 9 January 2020 for GPS specific serial data by Jim Giammanco, N5IB
 
 This example code is in the public domain.
 */

// <Wire.h> is a library generally needed for I2C and SPI protocols. 
//#include <Wire.h>  // It is commented out here, since it is invoked from within one or more of these other libraries:

#include <LiquidCrystal_I2C.h>  //  library needed for I2C displays, replaces the IDE's built-in LCD library (which must be deleted)
#include <SparkFunBME280.h>     //  library to support the BME-280 environmental sensor module
#include <SoftwareSerial.h>     //  library for the additional sotware serial ports

// *******************global variables and other constant definitions*******************************

// user-defined operating parameters
int UTC_offset = -6;                 // local standard time offset from UTC (-6 hours is Central Standard)
byte dispInt = 12;                   // interval between Watch/Barometer display swaps
const float baro_fix = 0.6;          // fudge factor for barometer to correct for altitude, in hPa, can be plus or minus
bool disp_knots=true;                // 'true' to show speed in knots, 'false' to show mph
bool Navy_mode=true;                 // 'true to operate as nautical clock, 'false' for ordinary chiming clock
#define GPS_baud 4800                // data rate for ASCII data from GPS receiver
#define MP3_baud 9600                // data rate for serial data sent to MP3 player (should not have to be changed)
#define LCD_I2Caddr 0x27             // I2C slave address for LCD display interface (run I2C scan to confirm)
#define sensorA_addr 0x76            // I2C slave address for BME-280s sensor module A, if used (run I2C scan to confirm)
#define sensorB_addr 0x77            // I2C slave address for BME-280s sensor module B, if used (run I2C scan to confirm)

//other global variables and definitiona
#define mp3_RX 2  //NANO's D2 should connect to RX pin of the Serial MP3 Player module
#define mp3_TX 3  //NANO's D3 should connect to TX pin of the module  (not actually used, but connected)
#define BACKLIGHT_PIN 3          // digital ouput to control LCD backlight (if used)
#define gpsRXpin 11   // RX on NANO's digital pin 11 (connect to TX of GPS)
#define gpsTXpin 13   // TX on NANO's digital pin 13 (connect to RX of GPS) - defined on D13... but not connected or used in software
#define PPS 12        //  digital input for 1 pulse per second from GPS - optional use
static int8_t Send_buf[8] = {0} ;
unsigned long TrackDelay = 20000 ;   // save generic track durations for delays
byte MP3Command, MP3DataH, MP3DataL; // arguments for send command function
byte selDir, selFile;                // directory and file selection for MP3 file
float Longitude, Lattitude, Pressure, Temperature, Humidity, DewPoint, LastGlassPress;
long Latx, Lonx;
String GPS, MH_txt, TimeZone;
bool bellsDone = false;              // flag to indicate whether chimes are needed
int Hr_bell, Min_bell;               //integer valuse parsed from GPS's ASCII hour and minute
byte swapCntr = 0;                   // counter to determine when to swap display formats
bool dispFlag = true;                // flag to indicate which display format is in use

// assign these these six digital I/O pins that form the jumper selection header JP2
#define JP2_0 9 //  spare, if not used to control AF amp power
#define JP2_1 8 //  enable/disable chimes               
#define JP2_2 7 //  display standard or daylight time if local time display is chosen               
#define JP2_3 6 //  display UTC or local time
#define JP2_4 5 //  LSBit of selection of directory on uSD card             
#define JP2_5 4 //  MSBit of selection of directory on uSD card
#define JP2_6 3 //  spare if not used for to receive serial data from MP3 player (maybe use to select Nautical/Normal chimes ?)

/*    MP3 folder selection table   
      JP2-5  JP2-4    Folder Name     "0" means pin is grounded with a shorting jumper on the JP2 header block
       1      1         01
       1      0         02
       0      1         03
       0      0         04
*/

// assign these two pins that control power to the audio amp(s)
#define AFpwrR 10 // digital output to control power to right channel AF amp
#define AFpwrL 9 // digital output to control power to left channel AF amp; or spare digital I/O

// define the MP3 player's command codes
#define SINGLE_CYCLE_ON 0X00
#define SINGLE_CYCLE_OFF 0X01
#define DEV_TF 0X02
#define CMD_PLAY_W_INDEX 0X03
#define CMD_LOUDER 0X04
#define CMD_SOFTER 0X05
#define CMD_SET_VOLUME 0X06
#define CMD_SEL_DEV 0X09
#define CMD_SLEEP 0X0A
#define CMD_WAKEUP 0X0B
#define CMD_RESET 0X0C
#define CMD_PLAY 0X0D
#define CMD_PAUSE 0X0E
#define CMD_PLAY_W_FOLDER_FILENAME 0X0F
#define CMD_STOP 0X16
#define CMD_CYCLEPLAY_W_FOLDERNAME 0X17
#define CMD_SHUFFLE_PLAY 0X18
#define CMD_SINGLE_CYCLE 0X19
#define CMD_DAC 0X1A
#define CMD_PLAY_W_VOL 0X22

// ********************end of gobal variables and definitions**********************

// create the LCD object, and set the LCD I2C address
LiquidCrystal_I2C lcd(LCD_I2Caddr, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);  
SoftwareSerial gpsSerial(gpsRXpin, gpsTXpin); // create serial comms object for GPS
SoftwareSerial mp3Serial(mp3_TX, mp3_RX);     // create serial comms for MP3, set up RX and TX lines for the MP3 player
                                              // one directional data only, only mp3RX needed
//create two sensor objects, assumes "A" I2C address is default 0x76, "B" address is 0x77
//these address will be invoked later, for example the  "mySensorA.setI2CAddress(sensorA_addr)" statement
BME280 mySensorA;
BME280 mySensorB;
                                          
void setup() {
  // Open serial communications and wait for port to open:
  Serial.begin(9600);
  while (!Serial) {
    ;    // wait for serial port to connect. Needed for native USB port only
  }
  Serial.println("GPS Chiming Clock");  // for debugging

  // set the data rate for the GPS SoftwareSerial port
  gpsSerial.begin(GPS_baud);

  lcd.begin(20, 4);  // configure for 20 X 4 display

  // Switch on the backlight
  pinMode ( BACKLIGHT_PIN, OUTPUT );
  digitalWrite ( BACKLIGHT_PIN, HIGH );

  lcd.home ();                   // go home
  lcd.clear();

  pinMode (JP2_4, INPUT_PULLUP); // jumper JP2-4 and JP2-5 select which directory to fetch MP3
  pinMode (JP2_5, INPUT_PULLUP);
  pinMode (JP2_2, INPUT_PULLUP); // jumper J2 selects standard or daylight time
  pinMode (JP2_3, INPUT_PULLUP); // jumper J3 selects local or UTC
  pinMode (JP2_1, INPUT_PULLUP); // enable/disable chimes
  pinMode (JP2_6, INPUT_PULLUP); // select nautical or normal chimes (assumes no Rx data from MP3 player)
  pinMode (AFpwrR,OUTPUT);     // drive this pin low to power up the right channel AF amp
  pinMode (AFpwrL,OUTPUT);     // drive this pin low to power up the left channel AF amp
  digitalWrite(AFpwrR,LOW); digitalWrite(AFpwrL, LOW);  // turn ON both the AF amps 
   
  lcd.setCursor(1,0);    
  lcd.print("Chiming Clock  N5IB"); //splash screen
  lcd.setCursor(1,2);    
  lcd.print("rev 3.02  24Jan2020"); //splash screen
  lcd.home ();                   // go home

   // read the jumpers to determine from which uSD card folder to fetch MP3 files
   selDir = 4 - (2*digitalRead(JP2_5)+digitalRead(JP2_4)); // directory 01,02,03, or 04
   Serial.print("Directory: "); Serial.println(selDir);  // for debugging
   
   mp3Serial.begin(MP3_baud);  // set the data rate for thee MP3 software serial port
   delay(500);//Wait chip initialization is complete
   sendCommand(CMD_SEL_DEV, 0x00, DEV_TF);//select the TF card  
   delay(200);//wait for 200ms
   sendCommand(CMD_SET_VOLUME, 0x00, 0x16);//set the playback volume,hex001E is full volume
   delay(200);//wait for 200ms
   sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir, 27); //  file 027 should be the sign-on sound in each folder

   delay(10000);   lcd.clear();  // wait for sign-on sound to finish
   digitalWrite(AFpwrR,HIGH);  digitalWrite(AFpwrL,HIGH);// turn OFF both the AF amps

//   mySensorA.setI2CAddress(sensorA_addr); //set BME-280(A) I2C address to default 0x76, note: Sparkfun BME280 hardware defaults to 0x77
//  if (mySensorA.beginI2C() == false) //Begin communication over I2C,
//  {
//    Serial.println("BME280A ERR");
//    while(1); //Freeze if I2C comms to sensor A are not working
//  }
//  delay(100);
  
  mySensorB.setI2CAddress(sensorB_addr); //optional: set BME-280(B) I2C address to 0x77, note: Sparkfun BME280 hardware defaults to 0x77
  if (mySensorB.beginI2C() == false) //Begin communication over I2C,
  {
    Serial.println("BME280B ERR");
    while(1); //Freeze if I2C comms to sensor B are not working
  }
   Serial.println("BME280 OK"); //other wise confirm valid communications with the sensor

   getWeather();  //  read pressure, temperature, and humidity from BME-280
   LastGlassPress=Pressure;  //initialize a barometer reading for comparison, will update each half hour

   if (digitalRead(JP2_6)) {  // read configuration jumper to select which chiming mode
     Navy_mode=true;
   }
   else {
     Navy_mode=false;
   }
   
} // end of setup()

void loop() { // run over and over
   gpsSerial.listen();  // enable GPS port - only one software serial port can recieve at any time
   GPS=gpsSerial.readStringUntil(0x0D); // wait for a $GPGGA sentence
   if (GPS.substring(1,7) == "$GPGGA")  {
     Serial.println(GPS);
     //lcd.clear();
     lcd.setCursor(0,0);   

      GetTimeZone();  // watch names and bells are always based on local time, even if UTC is being displayed
      if (digitalRead(JP2_2)) Hr_bell = ((GPS.substring(8,10)).toInt()) + UTC_offset;  // open-jumper: convert UTC to standard time (hours)
      if (!digitalRead(JP2_2)) Hr_bell = ((GPS.substring(8,10)).toInt()) + UTC_offset + 1;  // grounded jumper: convert UTC to daylight time (hours)
      if(Hr_bell < 0) Hr_bell=Hr_bell + 24;  // correct for rollover at UTC midnight   
      Min_bell = (GPS.substring(10,12)).toInt(); // minutes
        
      if (!digitalRead(JP2_3)) {      // grounded jumper: display GPS, UTC time as HH:MM  UTC
        lcd.print(GPS.substring(8,10));
        lcd.print(":"); 
        lcd.print(GPS.substring(10,12)); 
        lcd.print(" UTC");
      }
      else {     // open jumper: alternatively, display local time
//        GetTimeZone();
//        if (digitalRead(JP2_2)) Hr_bell = ((GPS.substring(8,10)).toInt()) + UTC_offset;  // convert UTC to standard time
//        if (!digitalRead(JP2_2)) Hr_bell = ((GPS.substring(8,10)).toInt()) + UTC_offset + 1;  // convert UTC to daylight time
//        if(Hr_bell < 0) Hr_bell=Hr_bell + 24;  // correct for rollover at UTC midnight   
//        Min_bell = (GPS.substring(10,12)).toInt();
         if (Hr_bell < 10) lcd.print("0"); // insert a leading zero for hours less than 10
         lcd.print(Hr_bell,DEC);
         lcd.print(":"); 
         lcd.print(GPS.substring(10,12)); // just use the ASCII from the GPS for the minutes
         lcd.print(TimeZone);
      }
          
     Serial.print(Hr_bell); Serial.print("  "); Serial.println(Min_bell); // for debugging

     // display lattitude as decimal degrees
     int latDeg = GPS.substring(19,21).toInt();
     float latFrac = GPS.substring(21,28).toFloat();
     Lattitude = latDeg+latFrac/60;
     lcd.setCursor(0,2);  
     lcd.print(Lattitude,4);
     lcd.write(0xDF); //degree symbol
     lcd.print(GPS.substring(29,30)); lcd.print("    "); // extra spaces to be sure to clear leftover characters
         
     // display longitude as decimal degrees
     int lonDeg = GPS.substring(31,34).toInt();
     float lonFrac = GPS.substring(34,41).toFloat();
     Longitude=lonDeg+lonFrac/60;
     if (Longitude < 100.0) {  //right justify the Lat/Lon display
        lcd.setCursor(11,2);  // for 10-99.9999
     }
     else if (Longitude<10.0) {  //for 0-9.9999
        lcd.setCursor(12,2); 
     }
     else {
         lcd.setCursor(10,2);  // for 100-180.0000
     }
     lcd.print(Longitude,4);
     lcd.write(0xDF); //degree symbol
     lcd.print(GPS.substring(42,43));
   
     MaidenHead(); // decode grid square 6-character designator

     //display status of GPS fix
     lcd.setCursor(14,0);
     if (GPS.substring(44,45) != "0") lcd.print(MH_txt); // display grid locator if the fix is valid
     if (GPS.substring(44,45) == "0") lcd.print("NO FIX"); // other wise do not show grid
     
     if (Navy_mode) {
       NavyBells();  //****** Work out the chimes and watches based on the hour and minute from the GPRMC sentence **********      
     }
     else {
       ChimingClock();   // ordinary chiming clock - quarter hours and hours 
     }
 
   } // end of $GPGGA processing
   
   gpsSerial.listen(); // enable GPS serial port to receive data
   GPS=gpsSerial.readStringUntil(0x0D); // wait for a $GPRMC sentence
   if (GPS.substring(1,7) == "$GPRMC")  {
     Serial.println(GPS);
     byte m = max(GPS.indexOf("W")+2, GPS.indexOf("E")+2);   // find either the letter "W" or "E" in the string
                     // take the larger of the two returns, since  -1 is returned if tha character is not found
     byte n = GPS.indexOf(",",m); // find the next comma, starting 2 characters after the "W" or "E"
     float VOG = GPS.substring(m,n).toFloat();  // that's where the speed lies
     m = n+1; // move past the comma that follows the speed
     n = GPS.indexOf(",",m); // find the next comma
     float Course=GPS.substring(m,n).toFloat(); // get the TRUE heading
     Serial.print(VOG); Serial.print("kts,  "); Serial.print(Course); Serial.println(" deg True");  // for debugging
     lcd.setCursor(0,3);          // cursor position for speed display
     if (disp_knots) {            // speed in nautical miles per hour (knots)
        lcd.print(VOG,1);         // display speed over ground (or water) in knots
        lcd.print("kts  ");
     }
     else {                       // speed in statute miles per hour
        lcd.print(VOG*1.15078,1); // convert NMEA's knots to mph and display
        lcd.print("mph");
     }
       lcd.setCursor(10,3);  // cursor location for course heading
//     if (VOG > 0.4) {      // if  this line and the ones marked *** are uncommented, no course will display for very slow speeds
        if(Course < 10) lcd.print("00");  // fix up leading zeros if needed
        else if(Course < 100) lcd.print("0");
        lcd.print(Course,0);
        lcd.write(0xDF); //degree symbol
        lcd.print(" True ");
//     }                             //   ***   needs to be uncommented if the above "if" statement is uncommented
//     else {                        //   ***   needs to be uncommented if the above "if" statement is uncommented
//        lcd.print(" - - - - -");   //   ***   needs to be uncommented if the above "if" statement is uncommented
//     }                             //   ***   needs to be uncommented if the above "if" statement is uncommented

   }    // end of $GPRMC processing 

   swapCntr = swapCntr + 1;       // count each time through the LCD display update loop
   if (swapCntr >= dispInt)       // after every 5 passes
   {
     dispFlag = !dispFlag;         // toggle the flag that determines the display format (Watch name, or Barometer)
     swapCntr = 0;                  // and reset the counter
   }
    
}       // end of loop()

// ****************************************************************************************
// routine to decode the 6-character Maidenhead grid locator based upon the current Lat/Lon
void MaidenHead(){
   // convert longitude into a long integer by multiplying by 100,000
   if (GPS.substring(42,43)=="W") Lonx = (-1)*Longitude*1e5;
      else Lonx=Longitude*1e5;
   Lonx = Lonx + 18000000;  // always positive east from Greenwich
   // Serial.print(Longitude); Serial.print("  "); Serial.println(Lonx);  // for debugging
   
   // convert lattitude into a long integer by multiplying by 100,000
   if (GPS.substring(29,30)=="S") Latx= (-1)*Lattitude*1e5;
      else Latx=Lattitude*1e5;
   Latx = Latx +  9000000; // Adjust so grid field AA is at the pole
   // Serial.print(Lattitude); Serial.print("  "); Serial.println(Latx);  // for debugging
   
   char MH[6] = {'A', 'A', '0', '0', 'a', 'a'};     // Initialise our print string
    MH[0] +=  Lonx / 2000000;                       // Field
    MH[1] +=  Latx / 1000000;
    MH[2] += (Lonx % 2000000) / 200000;             // Square
    MH[3] += (Latx % 1000000) / 100000;
    MH[4] += (Lonx %  200000) /   8333;             // Subsquare .08333 is   5/60 
    MH[5] += (Latx %  100000) /   4166;             //           .04166 is 2.5/60
  
   MH_txt = "";          // Build up Maidenhead, start with a null string
   int i = 0;            // add a character at a time, making a string that's easy to print
   while (i < 6){ 
      MH_txt += MH[i];
      i++; }

   Serial.print("Grid : ");
   Serial.println(MH_txt);

   return;
}  // end of maidenhead grid routine

//*************************************************************************
// routine to transmit a 7-byte command sequence serially to the MP3 module
void sendCommand(byte P1, byte P2, byte P3)
{
  delay(20);
  Send_buf[0] = 0x7e;               //starting byte
  Send_buf[1] = 0xff;               //version
  Send_buf[2] = 0x06;               //the number of bytes of the command without starting byte and ending byte
  Send_buf[3] = P1;                 // actual command code
  Send_buf[4] = 0x00;               //0x00 = no feedback, 0x01 = feedback
  Send_buf[5] = P2;                 //data-most sig byte
  Send_buf[6] = P3;                 //data-least sig byte
  Send_buf[7] = 0xef;               //ending byte
  for(byte i=0; i<8; i++)
  {
    mp3Serial.write(Send_buf[i]) ;  // transmit one byte at a time
  }
}  // end of command transmission routine

//*****************************************************************************
// routine to determine the time zone abbreviation based on the offset from UTC
void GetTimeZone()
{
   switch (UTC_offset) {  // the UTC offset was specified within the global variables
     case 12:
       TimeZone=(" UTC+12");
       break;  
     case 11:
       TimeZone=(" UTC+11");
       break;  
     case 10:
       TimeZone=(" UTC+10");
       break;  
     case 9:
       TimeZone=(" UTC+9");
       break;  
     case 8:
       TimeZone=(" UTC+8");
       break;  
     case 7:
       TimeZone=(" UTC+7");
       break;  
     case 6:
       TimeZone=(" UTC+6");
       break;  
     case 5:
       TimeZone=(" UTC+5");
       break;  
     case 4:
       TimeZone=(" UTC+4");
       break;  
     case 3:
       TimeZone=(" UTC+3");
       break;  
     case 2:
       TimeZone=(" UTC+2");
       break;  
     case 1:
       TimeZone=(" UTC+1");
       break;  
     case 0:
       TimeZone=(" UTC");
       break;  
     case -1:
       TimeZone=(" UTC-1");
       break;  
     case -2:
       TimeZone=(" UTC-2");
       break;  
     case -3:
       TimeZone=(" UTC-3");
       break;  
     case -4:
       TimeZone=(" UTC-4");
       break;  
     case -5:
       if (!digitalRead(JP2_2)) { // check whether daylight time is enabled
          TimeZone=(" EDT");    // if so, make it E"D"T
       }
       else {
          TimeZone=(" EST");    // elsewise, just standard time    
       }
       break;       
     case -6:
       if (!digitalRead(JP2_2)) {
          TimeZone=(" CDT");
       }
       else {
          TimeZone=(" CST");        
       }
       break;       
     case -7:
       if (!digitalRead(JP2_2)) {
          TimeZone=(" MDT");
       }
       else {
          TimeZone=(" MST");       
       }
       break;
     case -8:
       if (!digitalRead(JP2_2)) {
          TimeZone=(" PDT");
       }
       else {
          TimeZone=(" PST");        
       }
       break;                       
     case -9:
       if (!digitalRead(JP2_2)) {
       TimeZone=(" UTC-8");
//       TimeZone=(" AKDT");  // alternatively named Alaska Daylight Savings Time
       }
       else {
          TimeZone=(" UTC-9"); 
//          TimeZone=(" AKST");  // alternatively named Alaska Standard Time  
       }
       break;       
     case -10:
       TimeZone=(" UTC-10");
//       TimeZone=(" HST");  // alternativly named Hawaii Standard Time... daylight savings time is never used
       break;       
     case -11:
       TimeZone=(" UTC-11");
       break;       
     case -12:
       TimeZone=(" UTC-12");
       break;  
     default:
       TimeZone=(" ???   ");
       break; 
  }              
}  // end time zone routine

//*********************************************
// routine to work out the names of the watches
void getWatch()
{
   lcd.setCursor(0,1);
   switch (Hr_bell) {  // select the watch names based on the (local time) hour
     case 0:
       lcd.print("**** MID  WATCH ****");
       //       ("01234567890123456789")     character alignment guide
       break;
     case 1:
       lcd.print("**** MID  WATCH ****");
       break;
     case 2:
       lcd.print("**** MID  WATCH ****");
       break;
     case 3:
       lcd.print("**** MID  WATCH ****");
       break;
     case 4:
       lcd.print("** MORNING  WATCH **");
       break;
     case 5:
       lcd.print("** MORNING  WATCH **");
       break;
     case 6:
       lcd.print("** MORNING  WATCH **");
       break;
     case 7:
       lcd.print("** MORNING  WATCH **");
       break;
     case 8:
       lcd.print("** FORENOON WATCH **");
       break;
     case 9:
       lcd.print("** FORENOON WATCH **");
       break;
     case 10:
       lcd.print("** FORENOON WATCH **");
       break;
     case 11:
       lcd.print("** FORENOON WATCH **");
       break;
     case 12:
       lcd.print("* AFTERNOON  WATCH *");
       break;
     case 13:
       lcd.print("* AFTERNOON  WATCH *");
       break;
     case 14:
       lcd.print("* AFTERNOON  WATCH *");
       break;
     case 15:
       lcd.print("* AFTERNOON  WATCH *");
       break;
     case 16:
       lcd.print("* FIRST  DOG WATCH *");
       break;
     case 17:
       lcd.print("* FIRST  DOG WATCH *");
       break;
     case 18:
       lcd.print("* SECOND DOG WATCH *");
       break;
     case 19:
       lcd.print("* SECOND DOG WATCH *");
       break;
     case 20:
       lcd.print("** EVENING  WATCH **");
       break;
     case 21:
       lcd.print("** EVENING  WATCH **");
       break;
     case 22:
       lcd.print("** EVENING  WATCH **");
       break;
     case 23:
       lcd.print("** EVENING  WATCH **");
       break;    
     default:
       break;
   }
}    //   end of getWatch()

//***********************************************************************************
// routine to get weather data from BME-280 Pressure, Temperature, Humidity, DewPoint
void getWeather()  // read BME-280 sensor - Module B in use at this time
{
  // should read trmperature first to update compensators for other data
  // read temperature
  Temperature = mySensorB.readTempF(); //read thermometer
  
  // read pressure sensor
  Pressure = mySensorB.readFloatPressure()/100 + baro_fix; // read barometer in hecto pascals, hPa, correct for HASL
  // Pressure=Pressure * 0.02953;  // if desired, convert hPa to inches of mercury

  Serial.print("T="); Serial.print(Temperature,1); Serial.print("  P=");Serial.print(Pressure,1); // for debugging
  
  // read relative humidity, if desired
  //Humidity = mySensorB.readFloatHumidity(); //read humidity sensor
 
  // read dew point, if desired
  //DewPoint= mySensorB.dewPointF(); //calculate dew point in deg F

   return;
}    //       end of GetWeather()

//*****************************************************************************************
//routine to sound the ship's bell on the half hours, pipe the watches, and show baraometer
void NavyBells()
{
if (Min_bell==0 && !bellsDone) // Bells on the hour
     {
        if (digitalRead(JP2_1)) {  // open jumper: chimes are enabled, grounded jumper = disable
           digitalWrite(AFpwrR,LOW);  digitalWrite(AFpwrL,LOW);// turn ON both the AF amp control pins
        }
        LastGlassPress=Pressure;  // update  the comparison pressure
        if (Hr_bell==0 || Hr_bell==4 || Hr_bell==8 || Hr_bell==12 || Hr_bell==16 || Hr_bell==20) {
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,0x08); bellsDone=true; //play 8 Bells from directory 04
          delay(11500);  // wait for bells to complete
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,0x09); //bo'sun pipes relieve the watch         
        }
        else if (Hr_bell==1 || Hr_bell==5 || Hr_bell==9 || Hr_bell==13 || Hr_bell==17 || Hr_bell==21) {
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,0x02); bellsDone=true; //play 2 Bells from directory 04
        }
        else if (Hr_bell==2 || Hr_bell==6 || Hr_bell==10 || Hr_bell==14 || Hr_bell==18 || Hr_bell==22) {
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,0x04); bellsDone=true; //play 4 Bells from directory 04
          if (Hr_bell==18) {
            delay(5500); // wait for bells to complete
            sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,0x0F); //Roast Beef of Old England
          }
        }
        else if (Hr_bell==3 || Hr_bell==7 || Hr_bell==11 || Hr_bell==15 || Hr_bell==19 || Hr_bell==23) {
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,0x06); bellsDone=true; //play 6 Bells from directory 04
        }
     }
     else if (Min_bell==30 && !bellsDone)  // and bells on the half-hours
     {
        digitalWrite(AFpwrR,LOW);  digitalWrite(AFpwrL,LOW);  // turn on both the AF amps
        LastGlassPress=Pressure;  // update  the comparison pressure
        if (Hr_bell==0 || Hr_bell==4 || Hr_bell==8 || Hr_bell==12 || Hr_bell==16 || Hr_bell==20) {
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,0x01); bellsDone=true; //play 1 Bell from directory 04
        }
        else if (Hr_bell==1 || Hr_bell==5 || Hr_bell==9 || Hr_bell==13 || Hr_bell==17 || Hr_bell==21) {
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,0x03); bellsDone=true; //play 3 Bells from directory 04
        }
        else if (Hr_bell==2 || Hr_bell==6 || Hr_bell==10 || Hr_bell==14 || Hr_bell==18 || Hr_bell==22) {
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,0x05); bellsDone=true; //play 5 Bells from directory 04
        }
        else if (Hr_bell==3 || Hr_bell==7 || Hr_bell==11 || Hr_bell==15 || Hr_bell==19 || Hr_bell==23) {
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,0x07); bellsDone=true; //play 7 Bells from directory 04
        }
     }
     if (Min_bell==1 || Min_bell==31) {
       bellsDone=false; // after elapsed minutes, reset flag so bells will sound next half hour
       digitalWrite(AFpwrR,HIGH);  digitalWrite(AFpwrL,HIGH);   // turn OFF both the AF amps  
     }

     if (dispFlag) getWatch();  // on a timed cycle, go figure out which watch it is
     else {                     // otherwise, display the barometric pressure
        getWeather();
        float deltaP = Pressure - LastGlassPress;  // change since last "turning of the glass" at the hour or half hour
        lcd.setCursor(0,1);
        lcd.print("  "); lcd.print(Pressure,1); lcd.print(" hPa");
        if (deltaP >= 0.05) { lcd.print(", +"); lcd.print(deltaP,1); lcd.print("  ");
        }
        else if(deltaP <= -0.05) {  lcd.print(", "); lcd.print(deltaP,1); lcd.print("  ");
        }
        else { lcd.print("  ----  ");
        }
     }
}        // end of Navy bells processing    

//********************************************************************************
// routine to do normal clock chimes - mark the quarter hours and strike the hours
void ChimingClock()
{
  if (!bellsDone && Min_bell==15) { // to chime the first quarter hour
      digitalWrite(AFpwrR,LOW);  digitalWrite(AFpwrL,LOW);  // turn on both the AF amps
      sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,24); bellsDone=true; // play chime for first qharter hour
      delay(7000);  // wait for bells to complete 
  }

  if (!bellsDone && Min_bell==30) {  // to chime the half hour
      digitalWrite(AFpwrR,LOW);  digitalWrite(AFpwrL,LOW);  // turn on both the AF amps
      sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,25); bellsDone=true; // play chime for half hour
      delay(7000);  // wait for bells to complete
  }
  
  if (!bellsDone && Min_bell==45) {  // to chime the three quarter hour
      sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,26); bellsDone=true; // play chime for three quarter hour
      delay(7000);  // wait for bells to complete
      digitalWrite(AFpwrR,LOW);  digitalWrite(AFpwrL,LOW);  // turn on both the AF amps  
  }

  if (!bellsDone && Min_bell==0) {  // to chime the hour and/or strike the time
     digitalWrite(AFpwrR,LOW);  digitalWrite(AFpwrL,LOW);  // turn on both the AF amps 
     LastGlassPress=Pressure;  // update  the comparison pressure - to determine change since last hour 
  
     switch (Hr_bell) {  // select the chimes base on the (local time) hour
        case 0:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,0); bellsDone=true; // play chime for midnight
          delay(7000);  // wait for bells to complete
          break;
        case 1:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,1); bellsDone=true; // play chime for 1 am
          delay(7000);  // wait for bells to complete
          break;
        case 2:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,2); bellsDone=true; // play chime for 2 am
          delay(7000);  // wait for bells to complete
          break;
        case 3:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,3); bellsDone=true; // play chime for 3 am
          delay(7000);  // wait for bells to complete
          break;
        case 4:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,4); bellsDone=true; // play chime for 4 am
          delay(7000);  // wait for bells to complete
          break;
        case 5:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,5); bellsDone=true; // play chime for 5 am
          delay(7000);  // wait for bells to complete
          break;
        case 6:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,6); bellsDone=true; // play chime for 6 am
          delay(7000);  // wait for bells to complete
          break;
        case 7:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,7); bellsDone=true; // play chime for 7 am
          delay(7000);  // wait for bells to complete
          break;
        case 8:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,8); bellsDone=true; // play chime for 8 am
          delay(7000);  // wait for bells to complete
          break;
        case 9:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,9); bellsDone=true; // play chime for 9 am
          delay(7000);  // wait for bells to complete
          break;
        case 10:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,10); bellsDone=true; // play chime for 10 am
          delay(7000);  // wait for bells to complete
          break;
        case 11:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,11); bellsDone=true; // play chime for 11 am
          delay(7000);  // wait for bells to complete
          break;
        case 12:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,12); bellsDone=true; // play chime for 12 noon
          delay(7000);  // wait for bells to complete
          break;
        case 13:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,13); bellsDone=true; // play chime for 1 pm
          delay(7000);  // wait for bells to complete
          break;
        case 14:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,14); bellsDone=true; // play chime for 4 pm
          delay(7000);  // wait for bells to complete
          break;
        case 15:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,15); bellsDone=true; // play chime for 3 pm
          delay(7000);  // wait for bells to complete
          break;
        case 16:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,16); bellsDone=true; // play chime for 4 pm
          delay(7000);  // wait for bells to complete
          break;
        case 17:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,17); bellsDone=true; // play chime for 5 pm
          delay(7000);  // wait for bells to complete
          break;
        case 18:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,18); bellsDone=true; // play chime for 6 pm
          delay(7000);  // wait for bells to complete
          break;
        case 19:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,19); bellsDone=true; // play chime for 7 pm
          delay(7000);  // wait for bells to complete
          break;
        case 20:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,20); bellsDone=true; // play chime for 8 pm
          delay(7000);  // wait for bells to complete
          break;
        case 21:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,21); bellsDone=true; // play chime for 9 pm
          delay(7000);  // wait for bells to complete
          break;
        case 22:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,22); bellsDone=true; // play chime for 10 pm
          delay(7000);  // wait for bells to complete
          break;
        case 23:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, selDir,23); bellsDone=true; // play chime for 11 pm
          delay(7000);  // wait for bells to complete
          break;    
        default:
          break;
     }
  }
  if (bellsDone && (Min_bell==1 || Min_bell==16 || Min_bell==31 || Min_bell==46)) {
     bellsDone=false; // after an elapsed minute, reset flag so bells will sound next interval
     digitalWrite(AFpwrR,HIGH);  digitalWrite(AFpwrL,HIGH);   // turn OFF both the AF amps
  }
   
  getWeather();  // read barometer, write current pressure and the change since the top of the last hour
  float deltaP = Pressure - LastGlassPress;  // change since the last "top of the hour"
  lcd.setCursor(0,1);
  lcd.print("  "); lcd.print(Pressure,1); lcd.print(" hPa");
  if (deltaP >= 0.05) // only show change if the delta is significant, either plus or minus
  { 
     lcd.print(", +"); lcd.print(deltaP,1); lcd.print("  "); // need to explicitly write the "+" sign
  }
  else if(deltaP <= -0.05) // if singnificant negative change, "-" sign is automatic
  {  
     lcd.print(", "); lcd.print(deltaP,1); lcd.print("  ");
  }
  else 
  { 
     lcd.print("  ----  ");  // blank display if insignificant change
  }
  
}   // end of ChimingClock routine

