/*
  GPS_RTC_Chiming__Multi_BIG_Clock_N5IB_v6_7r00.ino
  this is a 'beta test' version for use by experimenters
  Matches hardware schematic and PC boards v6.xx (SMT) and 7.00 (through hole)
  14 January 2021 

This version supports one channel of audio output

I2C bus protocol is used for:
  Barometric sensor (BME-280)
  Real Time Clock module (PCF-8563)
  LCD 4-line x 20-character display
  
Uses Software Serial to implement multple serial ports:
  MP3 player
  GPS receiver
  
Receiving from NMEA data from GPS, selecting a given NMEA sentence
and parsing the sentence to extract relevant data,
and display parsed data to an LCD panel
 
 This example code is in the public domain.
 */

#include <LiquidCrystal_I2C.h>  //  library needed for I2C displays, replaces the IDE's built-in LCD library (which must be deleted)
#include <SparkFunBME280.h>     //  library to support the BME-280 environmental sensor module
#include <SoftwareSerial.h>     //  library for the additional sotware serial ports
#include <TinyGPS++.h>          // library for decoding GPS data
TinyGPSPlus MyGPS;              // The TinyGPS++ object

#include <Rtc_Pcf8563.h>        // library to support the PCF8563 real time clock module
Rtc_Pcf8563 rtc;                // create a real time clock object, the clock should be set to UTC

//-----------------------------------------------------------|
// To incorporate the menu feature into other sketches,      |
// include the code in this box, as well as the routines     |
// CheckMenu(), TestBTNS(), TestBtn(byte BTN), DoMenu(),     |
// FetchSettings(), and SaveSettings()                       |
// Parameters can be renamed as desired                      |
// invoke the menu by calling the routine CheckMenu()        |
//                                                           |
#include <EEPROM.h>  // library to support EEPROM access     |
#define MenuBtn 5 // assign I/O pins to MENU and SEL buttons |
#define SelBtn 6  //                                         |
// be sure to include these two lines in setup() :           |
// pinMode(MenuBtn,INPUT_PULLUP);                            |
// pinMode(SelBtn, INPUT_PULLUP);                            |
byte ParmCnt = 14;  // number of settings parameters         |
int MaxParm=12; // max absolute value of a numeric parameter |
//  the array below must have as many elements   (ParmCnt)   |
//  as the array of names that follows                       |
byte Settings[] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0};   //        |
//                                                           |
String ParmName[] = {// array of names for selections  |
  "GPS?    ",    // use GPS for time instead of RTC         |
  "UTC?    ",    // UTC or local time                       |
  "UTC+/-: ",    // offset from UTC (-6 = Central)          |
  "DST?    ",    // whether to use daylight savings time    |
  "SECONDS?",    // whether to display seconds              |
  "LARGE?  ",    // time display only, with 1" digits       |
  "CHIMES? ",    // enable the MP3 chimes                   |
  "NAVY?   ",    // ship's bell chimes or conventional      |
  "KNOTS?  ",    // Display knots or mph                    |
  "FUDGE:  ",    // the daily RTC correction factor in sec  |
  "BARO?   ",    // enable a barometic pressure display     |
  "BARO+/-:",    // barometer correction, in tenths of mb   |
  "NavyMP3:",    // directory number that hold Navy chimes  |
  "alt MP3:",    // directory holding normal chimes         |
  };              //                                         |
//-----------------------------------------------------------|

String MoABR[] = {"JAN", "FEB", "MAR", "APR", "MAY", "JUN", 
   "JUL", "AUG", "SEP", "OCT", "NOV", "DEC"};  // array of month abbreviations

// *******************global variables and other constant definitions********

// user-defined operating parameters, controlled by pushbutton menu
//
#define UseGPS Settings[0]           // TRUE to use GPS for time reference, FALSE for real time clock module
#define ShowUTC Settings[1]          // TRUE to display UTC, FALSE for local time, with daylight savings if selected
int UTC_offset = -6;                 // local standard time offset from UTC (-6 hours is Central Standard) as integer
#define UTCdelta Settings[2]         // UTC offset saved as unsigned byte in Settings[] array
#define UseDST Settings[3]           // TRUE to display daylight savings time, only when local time is being displayed
#define ShowSEC Settings[4]          // TRUE to display seconds, FALSE for HH:MM only
#define ShowBIG Settings[5]          // TRUE to show only the time with large digits, FALSE for multi-field small digit display
#define ChimesON Settings[6]         // TRUE to sound the selected type of chimes
#define Navy_mode Settings[7]        // TRUE to operate as nautical clock, 'false' for ordinary chiming clock
#define disp_knots Settings[8]       // TRUE to show speed in knots, 'false' to show mph
#define RTCfudge Settings[9]         // seconds per 24 hours to tweak the RTC module -- range is +/-  12 sec
#define ShowBARO Settings [10]       // TRUE to enable the BME-280 and show barometric pressure
float baro_fix = 0.6;                // fudge factor for barometer to correct for altitude, in hPa, can be plus or minus
#define BAROfudge Settings[11]       // amount in tenths of mb to correct the barometer for local sea level pressure   
#define NavyMP3 Settings[12]         // the folder number (001, 002, etc) wherin to find the ship's bell sounds
#define ChimeMP3 Settings[13]        // folder for the normal chimes

#define GPS_baud 4800                // data rate for ASCII data from GPS receiver
#define MP3_baud 9600                // data rate for serial data sent to MP3 player (should not have to be changed)
#define LCD_I2Caddr 0x27             // I2C slave address for LCD display interface (run I2C scan to confirm)
#define sensorA_addr 0x76            // I2C slave address for BME-280s sensor module A, if used (run I2C scan to confirm)
#define sensorB_addr 0x77            // I2C slave address for BME-280s sensor module B, if used (run I2C scan to confirm)
#define MP3vol 0x10                  // default MP3 player volume, 0x1E is maximum

//other global variables and definitions
#define mp3_RX 2  //NANO's D2 should connect to RX pin of the Serial MP3 Player module
#define mp3_TX 3  //NANO's D3 should connect to TX pin of the module  (not actually used, but connected)
#define BACKLIGHT_PIN 3          // digital ouput to control LCD backlight (if used)
#define gpsRXpin 11   // RX on NANO's digital pin 11 (connect to TX of GPS)
#define gpsTXpin 13   // TX on NANO's digital pin 13 (connect to RX of GPS) - defined on D13... but not connected or used in software
#define PPS 12        //  digital input for 1 pulse per second from GPS - optional use

static int8_t Send_buf[8] = {0} ;
bool bellsDone = false;              // flag to indicate whether chimes are needed
bool dispFlag = true; bool dispclrd=false;                // flags for display processing
byte MP3Command, MP3DataH, MP3DataL; // arguments for send command function
byte selDir, selFile;                // directory and file selection for MP3 file
byte TempHr, TempMin, TempSec, LastWx=0;
byte swapCntr = 0;                   // counter to determine when to swap display formats
byte StartChar;                      // for large character display
byte dispInt = 15;                   // interval in seconds between Watch/Barometer display swaps, should be an even divisor of 60
int Hr_bell, Min_bell=61, Sec_bell, LocHr;    //integer values parsed from GPS's ASCII hour, minute (dummy value here), and second
long Latx, Lonx;
float Longitude = -91.1, Lattitude = 30.3; // default Lat/Lon in case GPS is not connected 
float Pressure, Temperature, Humidity, DewPoint, LastGlassPress, VOG, Course;
String GPS, MH_txt, TimeZone;

/*    MP3 folder selection table   
      JP2-5  JP2-4    Folder Name     "0" means pin is grounded with a shorting jumper on the JP2 header block
       1      1         01
       1      0         02
       0      1         03
       0      0         04
*/

// assign these two pins that control power to the audio amp(s)
#define AFpwr 10 // digital output to control power to right channel AF amp

// define the MP3 player's command codes
#define SINGLE_CYCLE_ON 0X00
#define SINGLE_CYCLE_OFF 0X01
#define DEV_TF 0X02
#define CMD_PLAY_W_INDEX 0X03
#define CMD_LOUDER 0X04
#define CMD_SOFTER 0X05
#define CMD_SET_VOLUME 0X06
#define CMD_SEL_DEV 0X09
#define CMD_SLEEP 0X0A
#define CMD_WAKEUP 0X0B
#define CMD_RESET 0X0C
#define CMD_PLAY 0X0D
#define CMD_PAUSE 0X0E
#define CMD_PLAY_W_FOLDER_FILENAME 0X0F
#define CMD_STOP 0X16
#define CMD_CYCLEPLAY_W_FOLDERNAME 0X17
#define CMD_SHUFFLE_PLAY 0X18
#define CMD_SINGLE_CYCLE 0X19
#define CMD_DAC 0X1A
#define CMD_PLAY_W_VOL 0X22

// ********************end of gobal variables and definitions**********************

// create the LCD object, and set the LCD I2C address
LiquidCrystal_I2C lcd(LCD_I2Caddr, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);  
SoftwareSerial gpsSerial(gpsRXpin, gpsTXpin); // create serial comms object for GPS
SoftwareSerial mp3Serial(mp3_TX, mp3_RX);     // create serial comms for MP3, set up RX and TX lines for the MP3 player
                                              // one directional data only, only mp3RX needed
//BME280 mySensorA;    // create one or two sensor objects, assumes "A" I2C address is default 0x76, "B" address is 0x77
BME280 mySensorB;    // these address will be invoked later, 
                     // for example the  "mySensorA.setI2CAddress(sensorA_addr)" statement

//****************************************************************
void setup() {
  // Open serial communications and wait for port to open:
  Serial.begin(9600);
  while (!Serial) {
    ;    // wait for serial port to connect. Needed for native USB port only
  }
  Serial.println(F("GPS Chiming Clock"));  // for debugging

  // set the data rate for the GPS SoftwareSerial port
  gpsSerial.begin(GPS_baud);

  lcd.begin(20, 4);  // configure for 20 X 4 display

 // Switch on the backlight, if a PWM pin is used to control brightness
 // pinMode ( BACKLIGHT_PIN, OUTPUT );
 // digitalWrite ( BACKLIGHT_PIN, HIGH );
  lcd.clear();
  lcd.home ();                   // go to upper left corner (char=0, line=0)
  
  pinMode (MenuBtn, INPUT_PULLUP);   pinMode (SelBtn, INPUT_PULLUP);  // for menu and select buttons
  pinMode (AFpwr,OUTPUT);     // drive this pin low to power up the AF amp
 
  lcd.setCursor(0,0);    
  lcd.print(F("N5IB Multi-Use Clock"));  // splash screen at power-up or reset, F("xxx") gets string from flash memory instead of RAM
  lcd.setCursor(0,2);    
  lcd.print(F("rev5.22  23 Apr 2020"));  // more splash screen
  delay(2000);
  
// enter a routine to set the RTC module, which will exit automatically if no action taken
// MM:DD:YY, day of week, and HH:MM will get set, SS will default to 00, so routine should be terminated at SS = 00

  rtc.setAlarm(99, 99, 99, 99);     // clear any standing alarms, so the "fudge" will not affect setting the seconds
  SetRTC();
  
  FetchSettings();               // get the settings array from EEPROM
  DoMenu();                      // then enter the menu setting sequence - long press of MENU button exits
  SaveSettings();

  CustomChars();                  // create the set of custom characters for the optional large digit display

//   Serial.print("Directory: "); Serial.println(selDir);       // for debugging
   
   mp3Serial.begin(MP3_baud);  // set the data rate for thee MP3 software serial port
   delay(500);//Wait chip initialization is complete
   sendCommand(CMD_SEL_DEV, 0x00, DEV_TF);//select the TF card  
   delay(200);//wait for 200ms
   sendCommand(CMD_SET_VOLUME, 0x00, MP3vol);//set the playback volume,hex001E is full volume
   delay(200);//wait for 200ms
   if (ChimesON)
   {
      digitalWrite(AFpwr,LOW); // turn ON the AF amp 
      if (Navy_mode) { sendCommand(CMD_PLAY_W_FOLDER_FILENAME, NavyMP3, 27); } //  file 027 should be the sign-on sound in each folder
      else { sendCommand(CMD_PLAY_W_FOLDER_FILENAME, ChimeMP3, 27); }
      delay(8000);   lcd.clear();  // wait 8 sec for sign-on sound to finish
      digitalWrite(AFpwr,HIGH);  // turn OFF the AF amp
   }
   else { lcd.clear(); }
  
   if(ShowBARO)   // if the barometric pressure option has been enabled in the Settings
   {
//    mySensorA.setI2CAddress(sensorA_addr); //set BME-280(A) I2C address to default 0x76, note: Sparkfun BME280 hardware defaults to 0x77
//    if (mySensorA.beginI2C() == false)     //Begin communication over I2C,
//    {
//       Serial.println("BME280A ERR");
//       while(1); //Freeze if I2C comms to sensor A are not working
//    }
//    delay(100);
      
      mySensorB.setI2CAddress(sensorB_addr); //optional: set BME-280(B) I2C address to 0x77, note: Sparkfun BME280 hardware defaults to 0x77
      if (mySensorB.beginI2C() == false) //Begin communication over I2C,
      {
         Serial.println(F("BME280B ER"));
         while(1); //Freeze if I2C comms to sensor B are not working
      }
      Serial.println(F("BME280 OK")); //other wise confirm valid communications with the sensor

      // the "BAROfudge" parameter is a byte, which is unsigned, but is has been stored as if it was a signed binary number
      // calculate an integer which will reflect the +/- nature of that byte
      baro_fix = float(BAROfudge);                          // assume the positive number
      if (baro_fix > 127)  { baro_fix = baro_fix - 256 ; }  // but account for the MS bit set in BAROfudge
      // The barometer correction must be a floating point value, and able to increment in tenths of a mb
      baro_fix = baro_fix/10;    // since the BAROfudge  parameter is stored as an integer number of tenths
      getWeather(); LastGlassPress=Pressure;// read pressure, temperature, and humidity from BME-280
  } // initializes barometer reading for comparison, will update each half hour  

  //    set an alarm on the real time clock to activate once each day, choose the UTC hour at which the corretion takes place
  //    at each alarm event the clock will be given a slight correction to account for its inaccuracy
  //    in main loop, the alarm flag     rtc.alarmActive()    will be polled to see if it has been set
  if (!UseGPS)  // only do this if a GPS is **NOT** being used as the time reference
  {
    rtc.setAlarm(0, 0, 99, 99);  // minute, hour, day of month, day of week   --->  '99' means no alarm set for these  
  }

} // end of setup()

//**************************************************************************
void loop()   // run over and over
{
   if(UseGPS)  { Get_GPS_UTC(); }         // get the time data from the GPS receiver
   else        { Get_RTC_UTC(); }         // get the time data from the real time clock module
       
   lcd.setCursor(0,0);                    // character 0 of line 0 - upper left hand corner  

   //    need the local time, since watch names and bells are always based on local time, even if UTC is being displayed
   // the "UTCdelta" parameter is a byte, which is unsigned, but is has been stored as if it was a signed binary number
   // calculate an integer which will reflect the +/- nature of that byte
   UTC_offset = UTCdelta;                                      // assume the positive number
   if (UTC_offset > 127)  { UTC_offset = UTC_offset - 256 ; }  // but account for the MS bit set in UTCdelta

//   Serial.print("UTC offset calculated: "); Serial.println(UTC_offset);   // for debugging
  
   GetTimeZone();    // use the offset from UTC to dtermine the mnemonic for the time zone (if in USA)
   LocHr = Hr_bell + UTC_offset;                 // convert UTC to standard time (hours)
   if (UseDST)   LocHr = LocHr + 1;  // convert UTC to daylight time (hours)
   if(LocHr < 0)  LocHr = LocHr + 24;      // correct for rollover to 00 at local midnight 
   if(LocHr > 23) LocHr = LocHr - 24;      // correct for rollover to 00 at local midnight   

   if ( ((Min_bell == 0) || (Min_bell == 30)) && (Sec_bell <= 1 ) && ShowBARO ) 
   {
      LastGlassPress = Pressure;  // to compare the current pressure with that at the last half hour mark 
   }

   if (ShowBIG)
   {   //************* the following block of code processes the "large digit" display of time only, with optional chimes
        // now that the time has been acquired, either from GPS or from RTC, prepare to display on LCD

     if (!ShowUTC) { TempHr = LocHr;  }    // use local time if UTC has not been selected  
     else          { TempHr = Hr_bell; }   // otherwise the UTC from either GPS or RTC
      
     byte TensHr = TempHr/10 ;  byte UnitHr = TempHr%10 ;      // set up individual digits for display
     byte TensMin = Min_bell/10 ;  byte UnitMin = Min_bell%10 ;  // by finding the integer part of the quotient,
     byte TensSec = Sec_bell/10 ;  byte UnitSec = Sec_bell%10 ;  // and the remainder, after division by ten  
     
     if ((TensSec==0) && (UnitSec<4))   // only need to clear the display at the top of each minute
     {
       if (dispclrd==false) lcd.clear();  dispclrd=true;  //clear LCD and set the flag
     }
     else
     {
        dispclrd = false;  // clear the flag
     }
   
     StartChar=0;                            //  initialize where the digits will start
     if (TensHr==1) StartChar=StartChar+1 ;  // compensate for narrow "1" character in order to center display
     if (UnitHr==1) StartChar=StartChar+1 ;  // depending on how many "1" digits will be displayed
     if (TensMin==1) StartChar=StartChar+1 ;
     if (UnitMin==1) StartChar=StartChar+1 ;
    
     //  Serial.println(StartChar);
      
     // Serial.print(TensHr);
     BigNums(StartChar,TensHr); 
     // Serial.print(UnitHr);
     BigNums(StartChar,UnitHr);
     //  Serial.print(" : ");
     BigNums(StartChar,10);  // display a ":"
    
     // Serial.print(TensMin);
     BigNums(StartChar,TensMin);
     //   Serial.print(UnitMin); Serial.print(" : ");
     BigNums(StartChar,UnitMin);
    
     // Serial.print(TensSec);
     // Serial.println(UnitSec);
    
     if (ShowSEC) 
     {
        lcd.setCursor(StartChar,0) ; lcd.print(TensSec);  // display the seconds as normal sized characters
        lcd.setCursor(StartChar+1,1) ; lcd.print(UnitSec);
     }
    
     lcd.setCursor(StartChar,3); // position to show the UTC/Local/DST indicator
     if(ShowUTC)
     {
       if (StartChar > 17)              // account for the widest possible hh:mm display
       {
          lcd.setCursor(StartChar-1,3);  // slide the "UTC" indicator to the left
       }
          lcd.print(F("UTC"));              // UTC           
     }

     else
     {
       if (StartChar > 17)     //  for time zones, abbreviate the indicator when the hh:mm display is at max width
       {
         lcd.setCursor(StartChar-1,3);  // slide the "UTC" indicator to the left
       }
         lcd.print(TimeZone.substring(0, 3));             // normal 3-char abbreviation       
     }         
   }  //************** end of "large digit" display processing *********************
     
   else
   {   //*************the following block of code processes the "small display" screen, which features ***********  
//                      Time, Naval watch name, Position, Course & Speed, Grid square, and Barometric Pressure  

// write the four display lines, content depends on whether GPS or RTC is the time reference, and if barometer in use
      LineZero();
      LineOne();  
      LineTwo();
      LineThree();

      if (Sec_bell % dispInt == 0) dispFlag=!dispFlag ;  // dispInt must be an even divisor of 60
             
   }  //***************************end of "small display" processing **********************

   if (ChimesON)          // play MP3 files at designated time intervals
   {
      if (Navy_mode) {    // for the nautical chiming, the program expects the proper MP3 files to be in NavyMP3
        NavyBells();      //****** Work out the chimes and watches based on the hour and minute from the GPRMC sentence **********      
      }
      else {              // for the conventional chimes it expects MP3 files to be in folder ChimesMP3
        ChimingClock();   // ordinary chiming clock - identifies the quarter hours and tells the hours 
      }
   }

// check if the alarm flag nas been set (if in non-GPS mode) since the last time though the main loop
   if (!UseGPS && rtc.alarmActive()) 
   {
      rtc.getDateTime();                             // update the time registers
    Serial.print(F("Alarm discovered at  ")); Serial.println(rtc.formatTime());
      while (abs(rtc.getSecond() - 30) > 15) {      // want to do this in the middle of a minute to avoid rollover problem     
         rtc.getDateTime();                          // update the time registers
         Serial.print(abs(rtc.getSecond() - 30)); Serial.println(rtc.formatTime());               // for debugging
         delay(1500);                                   // wait a bit if necessary to get in the middle of a minute
      }
      rtc.getDateTime();                    // update the time registers, now that it's the middle of a minute
//    Serial.print(rtc.formatTime());       // for debugging
      byte newMin = rtc.getMinute();
      byte newHr = rtc.getHour();
      byte newSec = rtc.getSecond();

      // the "RTCfudge" parameter is a byte, which is unsigned, but is has been stored as if it was a signed binary number
      // calculate a temporary integer which will reflect the +/- nature of that byte
      int fff = RTCfudge;
      if (fff > 127)  { fff = fff - 256 ; }  // this means that the MS bit of RTCfudge was set
       
      newSec = newSec + fff;               // fudge value "fff" can be plus or minus, up to 12 seconds
      rtc.setTime(newHr, newMin, newSec);  // bump the clock back or forward by up to 9 seconds
      rtc.resetAlarm();                    // reactivate the alarm for the same time as before
    Serial.print(F("  Clock fudged after alarm flag set, now  "));   // for debugging
    Serial.println(rtc.formatTime());                             // for debugging              
  }

   CheckMenu();  // check for a short press of the MENU button to enter the menu sequence
     
} // end of loop()

// ****************************************************************************************
// routine to decode the 6-character Maidenhead grid locator based upon the current Lat/Lon
void MaidenHead(){
   // convert longitude into a long integer by multiplying by 100,000
   Lonx=Longitude*1e5;
   Lonx = Lonx + 18000000;  // always positive east from Greenwich
   // Serial.print(Longitude); Serial.print("  "); Serial.println(Lonx);  // for debugging
   
   // convert lattitude into a long integer by multiplying by 100,000
   Latx=Lattitude*1e5;
   Latx = Latx +  9000000; // Adjust so grid field AA is at the pole

   // Serial.print(Lattitude); Serial.print("  "); Serial.println(Latx);  // for debugging
   
   char MH[6] = {'A', 'A', '0', '0', 'a', 'a'};     // Initialise our print string
    MH[0] +=  Lonx / 2000000;                       // Field
    MH[1] +=  Latx / 1000000;
    MH[2] += (Lonx % 2000000) / 200000;             // Square
    MH[3] += (Latx % 1000000) / 100000;
    MH[4] += (Lonx %  200000) /   8333;             // Subsquare .08333 is   5/60 
    MH[5] += (Latx %  100000) /   4166;             //           .04166 is 2.5/60
  
   MH_txt = "";          // Build up Maidenhead, start with a null string
   int i = 0;            // add a character at a time, making a string that's easy to print
   while (i < 6){ 
      MH_txt += MH[i];
      i++; }

//  Serial.print("Grid : ");  Serial.println(MH_txt); for debugging

   return;
}  // end of maidenhead grid routine

//*************************************************************************
// routine to transmit a 7-byte command sequence serially to the MP3 module
void sendCommand(byte P1, byte P2, byte P3)
{
  delay(20);
  Send_buf[0] = 0x7e;               //starting byte
  Send_buf[1] = 0xff;               //version
  Send_buf[2] = 0x06;               //the number of bytes of the command without starting byte and ending byte
  Send_buf[3] = P1;                 // actual command code
  Send_buf[4] = 0x00;               //0x00 = no feedback, 0x01 = feedback
  Send_buf[5] = P2;                 //data-most sig byte
  Send_buf[6] = P3;                 //data-least sig byte
  Send_buf[7] = 0xef;               //ending byte
  for(byte i=0; i<8; i++)
  {
    mp3Serial.write(Send_buf[i]) ;  // transmit one byte at a time
  }
}  // end of command transmission routine

//*****************************************************************************
// routine to determine the time zone abbreviation based on the offset from UTC
void GetTimeZone()
{
   switch (UTC_offset) {  // the UTC offset was specified within the global variables
     case 12:
       TimeZone=(F("+12"));
       break;  
     case 11:
       TimeZone=(F("+11"));
       break;  
     case 10:
       TimeZone=(F("+10"));
       break;  
     case 9:
       TimeZone=(F(" +9"));
       break;  
     case 8:
       TimeZone=(F(" +8"));
       break;  
     case 7:
       TimeZone=(F(" +7"));
       break;  
     case 6:
       TimeZone=(F(" +6"));
       break;  
     case 5:
       TimeZone=(F(" +5"));
       break;  
     case 4:
       TimeZone=(F(" +4"));
       break;  
     case 3:
       TimeZone=(F(" +3"));
       break;  
     case 2:
       TimeZone=(F(" +2"));
       break;  
     case 1:
       TimeZone=(F(" +1"));
       break;  
     case 0:
       TimeZone=(F("UTC"));
       break;  
     case -1:
       TimeZone=(F(" -1"));
       break;  
     case -2:
       TimeZone=(F(" -2"));
       break;  
     case -3:
       TimeZone=(F(" -3"));
       break;  
     case -4:
       TimeZone=(F(" -4"));
       break;  
     case -5:
       if (UseDST) { // check whether daylight time is enabled
          TimeZone=(F("EDT"));    // if so, make it E"D"T
       }
       else {
          TimeZone=(F("EST"));    // elsewise, just standard time    
       }
       break;       
     case -6:
       if (UseDST) {
          TimeZone=(F("CDT"));
       }
       else {
          TimeZone=(F("CST"));        
       }
       break;       
     case -7:
       if (UseDST) {
          TimeZone=(F("MDT"));
       }
       else {
          TimeZone=(F("MST"));       
       }
       break;
     case -8:
       if (UseDST) {
          TimeZone=(F("PDT"));
       }
       else {
          TimeZone=(F("PST"));        
       }
       break;                       
     case -9:
       if (UseDST) {
       TimeZone=(F(" -8"));
//       TimeZone=(F("ADT"));  // alternatively named Alaska Daylight Savings Time
       }
       else {
          TimeZone=(F(" -9")); 
//          TimeZone=(F("AST"));  // alternatively named Alaska Standard Time  
       }
       break;       
     case -10:
       TimeZone=(F("-10"));
//       TimeZone=(F("HST"));  // alternativly named Hawaii Standard Time... daylight savings time is never used
       break;       
     case -11:
       TimeZone=(F("-11"));
       break;       
     case -12:
       TimeZone=(F("-12"));
       break;  
     default:
       TimeZone=(F("???"));
       break; 
  }              
}  // end time zone routine

//*********************************************
// routine to work out the names of the watches

void getWatch()
{
   lcd.setCursor(0,1);
   switch (LocHr) {  // select the watch names based on the (local time, with or without daylight savings) hour
     case 0: case 1: case 2: case 3:
       //       ("01234567890123456789")     character alignment guide
       lcd.print(F("**** MID  WATCH ****"));
       break;
     case 4: case 5: case 6: case 7:
       lcd.print(F("** MORNING  WATCH **"));
       break;
     case 8: case 9: case 10: case 11:
       lcd.print(F("** FORENOON WATCH **"));
       break;
     case 12: case 13 : case 14: case 15:
       lcd.print(F("* AFTERNOON  WATCH *"));
       break;
     case 16: case 17:
       lcd.print(F("* FIRST  DOG WATCH *"));
       break;
     case 18: case 19:
       lcd.print(F("* SECOND DOG WATCH *"));
       break;
     case 20: case 21: case 22: case 23:
       lcd.print(F("** EVENING  WATCH **"));
       break;
     default:
       break;
   }
}    //   end of getWatch()

//***********************************************************************************
// routine to get weather data from BME-280 Pressure, Temperature, Humidity, DewPoint
void getWeather()  // read BME-280 sensor - Module B in use at this time
{
  if (Min_bell == LastWx) return;   // skip updating if less than a minute has elapsed
  // should read temperature first to update compensators for other data
  // read temperature
  Temperature = mySensorB.readTempF(); //read thermometer
  
  // read pressure sensor
  Pressure = mySensorB.readFloatPressure()/100 + baro_fix; // read barometer in hecto pascals, hPa, correct for HASL
  // Pressure=Pressure * 0.02953;  // if desired, convert hPa to inches of mercury

//  Serial.print("T="); Serial.print(Temperature,1); Serial.print("  P=");Serial.print(Pressure,1); // for debugging
  
  // read relative humidity, if desired
  //Humidity = mySensorB.readFloatHumidity(); //read humidity sensor
 
  // read dew point, if desired
  //DewPoint= mySensorB.dewPointF(); //calculate dew point in deg F
   LastWx=Min_bell;
   return;
}    //       end of GetWeather()

//*****************************************************************************************
//routine to sound the ship's bell on the half hours, pipe the watches, always uses **local time** even if the display is UTC
void NavyBells()
{
if (Min_bell==0 && !bellsDone) // Bells on the hour
     {
        if (ChimesON) {  // open jumper: chimes are enabled, grounded jumper = disable
           digitalWrite(AFpwr,LOW);  // turn ON the AF amp control pin
        }
        if (LocHr==0 || LocHr==4 || LocHr==8 || LocHr==12 || LocHr==16 || LocHr==20) {
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, NavyMP3,0x08); bellsDone=true; //play 8 Bells and bo'sun pipe from directory       
        }
        else if (LocHr==1 || LocHr==5 || LocHr==9 || LocHr==13 || LocHr==17 || LocHr==21) {
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, NavyMP3,0x02); bellsDone=true; //play 2 Bells from directory
        }
        else if (LocHr==2 || LocHr==6 || LocHr==10 || LocHr==14 || LocHr==18 || LocHr==22) {
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, NavyMP3,0x04); bellsDone=true; //play 4 Bells from directory
          if (Hr_bell==18) {
//            delay(5500); // wait for bells to complete
//            sendCommand(CMD_PLAY_W_FOLDER_FILENAME, NavyMP3,0x0F); //Roast Beef of Old England
          }
        }
        else if (LocHr==3 || LocHr==7 || LocHr==11 || LocHr==15 || LocHr==19 || LocHr==23) {
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, NavyMP3,0x06); bellsDone=true; //play 6 Bells from directory
        }
     }
     else if (Min_bell==30 && !bellsDone)  // and bells on the half-hours
     {
        digitalWrite(AFpwr,LOW);   // turn on the AF amp
        if (LocHr==0 || LocHr==4 || LocHr==8 || LocHr==12 || LocHr==16 || LocHr==20) {
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, NavyMP3,0x01); bellsDone=true; //play 1 Bell from directory
        }
        else if (LocHr==1 || LocHr==5 || LocHr==9 || LocHr==13 || LocHr==17 || LocHr==21) {
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, NavyMP3,0x03); bellsDone=true; //play 3 Bells from directory
        }
        else if (LocHr==2 || LocHr==6 || LocHr==10 || LocHr==14 || LocHr==18 || LocHr==22) {
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, NavyMP3,0x05); bellsDone=true; //play 5 Bells from directory
        }
        else if (LocHr==3 || LocHr==7 || LocHr==11 || LocHr==15 || LocHr==19 || LocHr==23) {
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, NavyMP3,0x07); bellsDone=true; //play 7 Bells from directory
        }
     }
     if (Min_bell==1 || Min_bell==31) {
       bellsDone=false; // after an elapsed minute, reset flag so bells will sound next half hour
       digitalWrite(AFpwr,HIGH);   // turn OFF the AF amp 
     }
}        // end of Navy bells processing    

//********************************************************************************
// routine to do normal clock chimes - mark the quarter hours and strike the hours
void ChimingClock()
{
  if (!bellsDone && Min_bell==15) { // to chime the first quarter hour
      digitalWrite(AFpwr,LOW); // turn on both the AF amps
      sendCommand(CMD_PLAY_W_FOLDER_FILENAME, ChimeMP3,24); bellsDone=true; // play chime for first qharter hour
  }

  if (!bellsDone && Min_bell==30) {  // to chime the half hour
      digitalWrite(AFpwr,LOW); // turn on both the AF amps
      sendCommand(CMD_PLAY_W_FOLDER_FILENAME, ChimeMP3,25); bellsDone=true; // play chime for half hour
  }
  
  if (!bellsDone && Min_bell==45) {  // to chime the three quarter hour
      digitalWrite(AFpwr,LOW); // turn on both the AF amps  
      sendCommand(CMD_PLAY_W_FOLDER_FILENAME, ChimeMP3,26); bellsDone=true; // play chime for three quarter hour
  }

  if (!bellsDone && Min_bell==0) {  // to chime the hour and/or strike the time
     digitalWrite(AFpwr,LOW);  // turn on both the AF amps 
     LastGlassPress=Pressure;  // update  the comparison pressure - to determine change since last hour 
  
     switch (LocHr) {  // select the chimes base on the (local time) hour
        case 0:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, ChimeMP3,0); bellsDone=true; // play chime for midnight
          break;
        case 1:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, ChimeMP3,1); bellsDone=true; // play chime for 1 am
          break;
        case 2:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, ChimeMP3,2); bellsDone=true; // play chime for 2 am
          break;
        case 3:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, ChimeMP3,3); bellsDone=true; // play chime for 3 am
          break;
        case 4:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, ChimeMP3,4); bellsDone=true; // play chime for 4 am
          break;
        case 5:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, ChimeMP3,5); bellsDone=true; // play chime for 5 am
          break;
        case 6:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, ChimeMP3,6); bellsDone=true; // play chime for 6 am
          break;
        case 7:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, ChimeMP3,7); bellsDone=true; // play chime for 7 am
          break;
        case 8:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, ChimeMP3,8); bellsDone=true; // play chime for 8 am
          break;
        case 9:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, ChimeMP3,9); bellsDone=true; // play chime for 9 am
          break;
        case 10:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, ChimeMP3,10); bellsDone=true; // play chime for 10 am
          break;
        case 11:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, ChimeMP3,11); bellsDone=true; // play chime for 11 am
          break;
        case 12:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, ChimeMP3,12); bellsDone=true; // play chime for 12 noon
          break;
        case 13:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, ChimeMP3,13); bellsDone=true; // play chime for 1 pm
          break;
        case 14:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, ChimeMP3,14); bellsDone=true; // play chime for 4 pm
          break;
        case 15:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, ChimeMP3,15); bellsDone=true; // play chime for 3 pm
          break;
        case 16:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, ChimeMP3,16); bellsDone=true; // play chime for 4 pm
          break;
        case 17:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, ChimeMP3,17); bellsDone=true; // play chime for 5 pm
          break;
        case 18:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, ChimeMP3,18); bellsDone=true; // play chime for 6 pm
          break;
        case 19:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, ChimeMP3,19); bellsDone=true; // play chime for 7 pm
          break;
        case 20:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, ChimeMP3,20); bellsDone=true; // play chime for 8 pm
          break;
        case 21:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, ChimeMP3,21); bellsDone=true; // play chime for 9 pm
          break;
        case 22:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, ChimeMP3,22); bellsDone=true; // play chime for 10 pm
          break;
        case 23:
          sendCommand(CMD_PLAY_W_FOLDER_FILENAME, ChimeMP3,23); bellsDone=true; // play chime for 11 pm
          break;    
        default:
          break;
     }
  }
  if (bellsDone && (Min_bell==1 || Min_bell==16 || Min_bell==31 || Min_bell==46)) {
     bellsDone=false; // after an elapsed minute, reset flag so bells will sound next interval
     digitalWrite(AFpwr,HIGH); // turn OFF both the AF amps
  }

  if (ShowBARO)
  {
     getWeather();  // read barometer, write current pressure and the change since the top of the last hour
     float deltaP = Pressure - LastGlassPress;  // change since the last "top of the hour"
     lcd.setCursor(0,1);
     lcd.print(F("  ")); lcd.print(Pressure,1); lcd.print(F(" hPa"));
     if (deltaP >= 0.07) // only show change if the delta is significant, either plus or minus
     { 
        lcd.print(F(", +")); lcd.print(deltaP,1); lcd.print(F("  ")); // need to explicitly write the "+" sign
     }
     else if(deltaP <= -0.07) // if significant negative change, "-" sign is automatic
     {  
        lcd.print(F(", ")); lcd.print(deltaP,1); lcd.print(F("  "));
     }
     else 
     { 
        lcd.print(F("  ----  "));  // blank display if insignificant change
     }
  } 
}   // end of ChimingClock routine

//**************************************************************************
//routine to create the custom character set and store it in the LCD RAM
void CustomChars() // creates a set of custom characters to be used to make large size displays
{
  byte tempChar[8];  // an array to hold the bit maps for the special characters

  tempChar[0]=B00001; //  a "0" means a dark pixel, "1" is for an illuminated pixel
  tempChar[1]=B00011; //  each character is 5 pixels wide and 8 pixels high 
  tempChar[2]=B00111; 
  tempChar[3]=B01111;
  tempChar[4]=B11111;  
  tempChar[5]=B11111;
  tempChar[6]=B11111;
  tempChar[7]=B11111;
  lcd.createChar(0, tempChar); // also maps to address 8

  tempChar[0]=B11111;
  tempChar[1]=B11111;   
  tempChar[2]=B11111; 
  tempChar[3]=B11111;
  tempChar[4]=B01111;  
  tempChar[5]=B00111;
  tempChar[6]=B00011;
  tempChar[7]=B00001;
  lcd.createChar(1, tempChar); // also maps to address 9

  tempChar[0]=B11111;
  tempChar[1]=B11111;   
  tempChar[2]=B11111; 
  tempChar[3]=B11111;
  tempChar[4]=B00000;  
  tempChar[5]=B00000;
  tempChar[6]=B00000;
  tempChar[7]=B00000;
  lcd.createChar(2, tempChar);  // also maps to address 10
  
  tempChar[0]=B00000;
  tempChar[1]=B00000;   
  tempChar[2]=B00000; 
  tempChar[3]=B00000;
  tempChar[4]=B11111;  
  tempChar[5]=B11111;
  tempChar[6]=B11111;
  tempChar[7]=B11111;
  lcd.createChar(3, tempChar);   // also maps to address 11
  
  tempChar[0]=B00000;
  tempChar[1]=B00000;   
  tempChar[2]=B00000; 
  tempChar[3]=B00000;
  tempChar[4]=B00000;  
  tempChar[5]=B00000;
  tempChar[6]=B11111;
  tempChar[7]=B11111;
  lcd.createChar(4, tempChar);  // also maps to address 12 
  
  tempChar[0]=B11111;
  tempChar[1]=B11111;   
  tempChar[2]=B00000; 
  tempChar[3]=B00000;
  tempChar[4]=B00000;  
  tempChar[5]=B00000;
  tempChar[6]=B00000;
  tempChar[7]=B00000;
  lcd.createChar(5, tempChar);   // also maps to address 13

  tempChar[0]=B10000;
  tempChar[1]=B11000;   
  tempChar[2]=B11100; 
  tempChar[3]=B11110;
  tempChar[4]=B11111;  
  tempChar[5]=B11111;
  tempChar[6]=B11111;
  tempChar[7]=B11111;
  lcd.createChar(6, tempChar); // also maps to address 14
    
  tempChar[0]=B11111;
  tempChar[1]=B11111;   
  tempChar[2]=B11111; 
  tempChar[3]=B11111;
  tempChar[4]=B11110;  
  tempChar[5]=B11100;
  tempChar[6]=B11000;
  tempChar[7]=B10000;
  lcd.createChar(7, tempChar);   // also maps to address 15
}

//***************************************************************************
//routine to generate large numerical characters 0 through 9, plus a colon
void BigNums(byte CharPos, byte NumDisp) // character position  (0-19) to start, and number to display
{
   switch(NumDisp) {
     case 0: // each larger character is made up of a 3 wide x 4 high pattern of special characters, except for "1" and ":"
         lcd.setCursor(StartChar,0);
         lcd.write(8); lcd.write(2); lcd.write(6);  // "8" means the same as "0" here..... don't ask  %&$#*!
         lcd.setCursor(CharPos,1);
         lcd.write(0xFF); lcd.write(0x20); lcd.write(0xFF);
         lcd.setCursor(CharPos,2);
         lcd.write(0xFF); lcd.write(0x20); lcd.write(0xFF);
         lcd.setCursor(CharPos,3);
         lcd.write(1); lcd.write(3); lcd.write(7);
         StartChar=CharPos+4; // bump the position pointer to get ready for the next character
         break;
                        
     case 1:
         lcd.setCursor(CharPos,0);  
         lcd.write(8);
         lcd.setCursor(CharPos,1);
         lcd.write(0xFF);
         lcd.setCursor(CharPos,2);
         lcd.write(0xFF);
         lcd.setCursor(CharPos,3);
         lcd.write(0xFF);
         StartChar=CharPos+2;
         break;

     case 2:
         lcd.setCursor(CharPos,0);  
         lcd.write(2); lcd.write(2); lcd.write(6);
         lcd.setCursor(CharPos,1);
         lcd.write(0x20); lcd.write(4); lcd.write(7);
         lcd.setCursor(CharPos,2);
         lcd.write(8); lcd.write(5); lcd.write(0x20);
         lcd.setCursor(CharPos,3);
         lcd.write(0xFF); lcd.write(3); lcd.write(3);
         StartChar=CharPos+4;     
         break;

     case 3:
         lcd.setCursor(CharPos,0);  
         lcd.write(2); lcd.write(2); lcd.write(6);
         lcd.setCursor(CharPos,1);
         lcd.write(0x20); lcd.write(4); lcd.write(0xFF);
         lcd.setCursor(CharPos,2);
         lcd.write(0x20); lcd.write(5); lcd.write(0xFF);
         lcd.setCursor(CharPos,3);
         lcd.write(3); lcd.write(3); lcd.write(7);
         StartChar=CharPos+4;     
         break;

     case 4:
         lcd.setCursor(CharPos,0);  
         lcd.write(8); lcd.write(0x20); lcd.write(8);
         lcd.setCursor(CharPos,1);
         lcd.write(0xFF); lcd.write(4); lcd.write(0xFF);
         lcd.setCursor(CharPos,2);
         lcd.write(5); lcd.write(5); lcd.write(0xFF);
         lcd.setCursor(CharPos,3);
         lcd.write(0x20); lcd.write(0x20); lcd.write(0xFF);
         StartChar=CharPos+4;          
         break;

     case 5:
         lcd.setCursor(CharPos,0);  
         lcd.write(8); lcd.write(2); lcd.write(2);
         lcd.setCursor(CharPos,1);
         lcd.write(0xFF); lcd.write(3); lcd.write(0x20);
         lcd.setCursor(CharPos,2);
         lcd.write(0x20); lcd.write(0x20); lcd.write(0xFF);
         lcd.setCursor(CharPos,3);
         lcd.write(6); lcd.write(3); lcd.write(7);
         StartChar=CharPos+4;              
         break;

     case 6:
         lcd.setCursor(CharPos,0);  
         lcd.write(8); lcd.write(2); lcd.write(2);
         lcd.setCursor(CharPos,1);
         lcd.write(0xFF); lcd.write(4); lcd.write(4);
         lcd.setCursor(CharPos,2);
         lcd.write(0xFF); lcd.write(5); lcd.write(0xFF);
         lcd.setCursor(CharPos,3);
         lcd.write(1); lcd.write(3); lcd.write(7);
         StartChar=CharPos+4;         
         break;

     case 7:
         lcd.setCursor(CharPos,0);  
         lcd.write(8); lcd.write(2); lcd.write(0xFF);
         lcd.setCursor(CharPos,1);
         lcd.write(0x20); lcd.write(0x20); lcd.write(0xFF);
         lcd.setCursor(CharPos,2);
         lcd.write(0x20); lcd.write(0x20); lcd.write(0xFF);
         lcd.setCursor(CharPos,3);
         lcd.write(0x20); lcd.write(0x20); lcd.write(7);
         StartChar=CharPos+4;              
         break;

     case 8:
         lcd.setCursor(CharPos,0);  
         lcd.write(8); lcd.write(2); lcd.write(6);
         lcd.setCursor(CharPos,1);
         lcd.write(0xFF); lcd.write(4); lcd.write(0xFF);
         lcd.setCursor(CharPos,2);
         lcd.write(0xFF); lcd.write(5); lcd.write(0xFF);
         lcd.setCursor(CharPos,3);
         lcd.write(1); lcd.write(3); lcd.write(7);
         StartChar=CharPos+4;              
         break; 

     case 9:
         lcd.setCursor(CharPos,0);  
         lcd.write(8); lcd.write(2); lcd.write(6);
         lcd.setCursor(CharPos,1);
         lcd.write(0xFF); lcd.write(4); lcd.write(0xFF);
         lcd.setCursor(CharPos,2);
         lcd.write(0x20); lcd.write(5); lcd.write(0xFF);
         lcd.setCursor(CharPos,3);
         lcd.write(3); lcd.write(3); lcd.write(7);
         StartChar=CharPos+4;                   
         break;

     case 10:  // draws ":"
         lcd.setCursor(CharPos,1);  
         lcd.write(2);
         lcd.setCursor(CharPos,2);
         lcd.write(3);
         StartChar=CharPos+2;     
         break;                
   }  
}

//****************************************************************
// routine to test the menu button for a valid press
void CheckMenu()
{
  byte temp1 = TestBTNS();   // lcd.setCursor(10,3) ; lcd.print(temp1);
  if (temp1==1)  // short press of MENU button, so initiate the menu routine
  {
     DoMenu();  // uses two momentary, normally open, pushbuttons to GND to manipulate a settings menu
     SaveSettings();
     return;  
  }

  else if (temp1==10) { return; } // long press of MENU button is an escape
  
  else   { return; }
}

//**********************************************************************************************************
//routine to test all buttons and return a byte value indicating which was pressed, if any, and for how long
byte TestBTNS()
{
   unsigned long sTime=millis();  // start a timer
   if (TestBtn(MenuBtn))
   {
      if ((millis() - sTime) > 2000)  { return 10; }  // indicates a long press of the MENU button
      else { return 1; }                              // indicates a short press of the MENU button
   }

   else if (TestBtn(SelBtn))
   {
     if ((millis() - sTime) > 2000) { return 20; }   // indicates a long press of the SEL button     
     else { return 2; }                              //  indicates a short press of the SEL button
   }   
     else return 0;                                  // indicates that the was no valid button press
}
                   
//******************************************************************************************
//routine to test a single button and return a TRUE condition if there was a valid press
bool TestBtn(byte BTN)
{
   if (digitalRead(BTN)) return false;  // if not pressed, and pulled up HIGH, return without action
   delay(50); // if it was pressed, wait a bit then test again
   if (digitalRead(BTN)) return false;  // if now not pressed, was a false condition, so return
   delay(50); // if it was still pressed, wait a bit then test one more time
   if (digitalRead(BTN)) return false;  // if now not pressed, was a false condition, so return
   while (!digitalRead(BTN)) {  // it is still pressed, so
    // now wait for the button to be released     
   }
   return true;  // and report back that there was a valid press
}


//******************************************************************************************
//menu processing routine
void DoMenu()
{
   unsigned long start = millis();
   lcd.clear(); lcd.setCursor(0,0);
   lcd.print(F("Menu Select:"));  delay(100);
   for (byte i=0; i<=ParmCnt-1; i++)
   {
       lcd.setCursor(0,3); lcd.print(F("                    "));  // clear the bottom line
       byte tst=0;
       while (tst != 1)            // wait for another press of either button
       {
          lcd.setCursor(0,2);                            // position for text on next-to-last line
          lcd.print(ParmName[i]);                        // show the name of the parameter
          if (Settings[i] != 0) { lcd.print(F(" YES")); }   // display either "YES" 
          else { lcd.print(F(" NO ")); }                    // or "NO" to indicate if the given byte is "0" or "1"            

          tst=TestBTNS(); 
//          Serial.println(tst);  // for debugging
          if (tst==10) return;    // long press of MENU button is an escape back to the main routine
          if (millis() - start > 10000) return;  // or if no action in 10 seconds, return to main
          if (tst==1) { start = millis(); break; }     // short press of MENU button, so go on to next parameter, reset timer

          if (tst==2)            // a short press of the SEL button toggles the parameter between "YES" and "NO" 
          {
             start = millis();  // reset the timer
             Settings[i] = ( 1 ^ Settings[i]) ;  // SEL button was short-pressed, toggle the parameter          
          }

          if (tst==20)           // if a long-press of the SEL button invokes the alternate operation
          {
             start = millis();   // reset the timer
             // do something other than YES/NO toggling, such as selecting a numeric value
             byte tst2 = TestBTNS();
             int tset = Settings[i];  // fetch the current value, and make an integer to allow for negative numbers
             if (tset >= 128) tset = tset - 256;   // for twos complement, to simulate negative value stored in a byte variable
             while (tst2 != 1)       // another short press of MENU button will exit this part
             {
                lcd.setCursor(0,3); lcd.print(F("Value = ")); 
                lcd.setCursor(8,3); lcd.print(tset); lcd.print("   ");  // clear remainder of data field
                if (tst2 == 2)                                          // look for a short press of the SEL button
                {
                   tset = tset + 1;                  // increment the value of this parameter
                   if (tset > MaxParm) tset = -MaxParm;  //  set a limit, and allow negative values               
                }
                tst2 = TestBTNS();  // single short press of MENU exist this data entry part, another short press goes to next item
                start = millis();   // reset the timer  **************************************               
             }  // end of inner "while" structure
             Settings[i] = tset;   // update the setting, which will get saved later in EEPROM                
          }
          delay(5);
       }  // end of outer "while" structure
   }

}  // end of menu processing

//*****************************************************************************************
//routine to save settings in EEPROM
void SaveSettings()
{
// after each call to the Menu routine, save the (possibly modified) parameters array to the EEPROM 
   for (int SetAdr=0; SetAdr<=(ParmCnt-1) ; SetAdr++)     // using addresses 0 to 7
   {
      EEPROM.write(SetAdr, Settings[SetAdr]);   // save all values of Settings[] array to EEPROM
   }
   lcd.clear(); lcd.setCursor(4,2); lcd.print("EEPROM Saved"); delay(1500); lcd.clear();     
}

//*****************************************************************************************
// routine to retrieve settings from EEPROM
void FetchSettings()
{
  for (int SetAdr=0; SetAdr<=(ParmCnt-1) ; SetAdr++) // as many as addresses as the number of settings
  {
     Settings[SetAdr] = EEPROM.read(SetAdr);       // retrieve the Settings parameter array from EEPROM   
  } 
}

//*************************************************************************************
// routine to get UTC time from the GPS receiver using Software Serial communications
void Get_GPS_UTC()
{
  gpsSerial.listen();   // supplies information every time a new sentence is correctly encoded.

  unsigned long start = millis();
  do 
  {
    while (gpsSerial.available())
      MyGPS.encode(gpsSerial.read());
  } while (millis() - start < 650);

     if (MyGPS.time.isValid())
     {   
        Hr_bell = MyGPS.time.hour();
        Min_bell = MyGPS.time.minute();
        Sec_bell = MyGPS.time.second();
     }
     if (MyGPS.location.isValid())
     {
        Lattitude = MyGPS.location.lat();
        Longitude = MyGPS.location.lng();
//        Serial.print(Lattitude,4); Serial.print("  "); Serial.println(Longitude,4);  // for debugging
     } 
     if (MyGPS.speed.isUpdated())
     {
        VOG = MyGPS.speed.knots();  
     }
     if (MyGPS.course.isUpdated())
     {
        Course = MyGPS.course.deg();
     }
}

//*************************************************************
// routine to get UTC time from real time clock module
void Get_RTC_UTC()
{
   while (rtc.getSecond() == Sec_bell) { rtc.getDateTime(); } //update RTC time reporting registers
   Sec_bell = rtc.getSecond();
   Hr_bell = rtc.getHour();
   Min_bell = rtc.getMinute();
}

//******************************************************************************
// routine to write first line (Line 0) of LCD small digits display
void LineZero()
{
      if (ShowUTC)           // display GPS derived UTC ASCII time as "HH:MM UTC"
      {    
         if (Hr_bell < 10) lcd.print(F("0")); // insert a leading zero character for hours less than 10
         lcd.print(Hr_bell,DEC);
      }
      else           // alternatively, display local time, using the integers calculated above from the ASCII 
      {     
         if (LocHr < 10) lcd.print(F("0")); // insert a leading zero character for hours less than 10
         lcd.print(LocHr,DEC);
      }
      lcd.print(":"); 
      if (Min_bell < 10) lcd.print(F("0")); // insert a leading zero character for minutes less than 10
      lcd.print(Min_bell,DEC);

      if (ShowSEC)
      {
         lcd.print(":");
         if (Sec_bell < 10) lcd.print(F("0")); // insert a leading zero character for seconds less than 10
         lcd.print(Sec_bell,DEC);
      }
           
      if (!ShowUTC) { lcd.print(F(" ")); lcd.print(TimeZone); }   // append appropriate time zone mnemonic
      else { lcd.print(F(" UTC")); }

      lcd.setCursor(14,0);                                  // the upper right 6 characters of the display - right justified
      if( UseGPS)  //  If GPS is attached,  and the fix is valid, display the maidenhead grid square
      {
          MaidenHead(); // from the lattitude and longitude, decode the Maidenhead 6-character grid square designator    
         //display status of GPS fix
         if (MyGPS.location.isValid())  { lcd.print(MH_txt); }        // display the decoded grid locator only if the fix is valid 
         else                           { lcd.print(F("NO FIX")); }   // other wise do not show grid
      }
      else  // otherwise, get the ***UTC*** date from the RTC and display that
            // there's still some work to do to convert UTC date to local date based on UTC offset
      {
         lcd.print(rtc.getDay()); lcd.print(F(" ")); lcd.print(MoABR[rtc.getMonth()-1]);
      }
}

//***************************************************************************************************
// routine to write second line (Line 1) of small digits LCD display
void LineOne()
{
   if (dispFlag && Navy_mode) getWatch();  // if in nautical mode, on a timed cycle, go figure out which watch it is
   else {                                  // and also show the barometer, otherwise, only display the barometric pressure
      if (ShowBARO)
      {
        getWeather();
        float deltaP = Pressure - LastGlassPress;  // change since last "turning of the glass" at the hour or half hour
        lcd.setCursor(0,1);
         lcd.print(F("  ")); lcd.print(Pressure,1); lcd.print(F(" hPa"));
        if (deltaP >= 0.05) { lcd.print(F(", +")); lcd.print(deltaP,1); lcd.print(F("  "));
        }
        else if(deltaP <= -0.05) {  lcd.print(F(", ")); lcd.print(deltaP,1); lcd.print(F("  "));
        }
        else { lcd.print(F("  ----  "));
        }
      }
   }
}

//***************************************************************************************
// routine to write the third line (Line 2) of small digits LCD display
void LineTwo()
{
   // Serial.print(Hr_bell); Serial.print("  "); Serial.println(Min_bell); // for debugging purposes
   // Serial.print(fabs(Lattitude),4); Serial.print("  "); Serial.println(fabs(Longitude),4);   // for debugging
   // display lattitude as decimal degrees, always as a positive number, with "N" or "S" indicator
   lcd.setCursor(0,2);                                      // position the cursor on line 2, left margin
   lcd.print(fabs(Lattitude),4);                             // display lattitude as decimal degrees to four places  DD.dddd 
   lcd.write(0xDF);                                    // append the degree symbol
   if (Lattitude < 0) { lcd.print(F("S    ")); } // write the "N" or "S" then some extra spaces to be sure to clear any leftover characters
   else { lcd.print(F("N    ")); }
    
   // in the same manner as above, display longitude as decimal degrees
   if (fabs(Longitude) < 100.0) {    //right justify the Lat/Lon display, want the "W" or "E" to be at the right hand margin of line 2
      lcd.setCursor(11,2);     // for 10-99.9999
   }
   else if (fabs(Longitude) < 10.0) {  //for 0-9.9999
      lcd.setCursor(12,2); 
   }
   else {
      lcd.setCursor(10,2);    // for 100-180.0000
   }
   lcd.print(fabs(Longitude),4);
   lcd.write(0xDF);            //degree symbol
   if (Longitude < 0) { lcd.print(F("W")); } // write the "E" or "W" 
   else { lcd.print(F("E")); }  
}

//****************************************************************************************
// routine to write last line (Line 3) of small digits LCD display
void LineThree()
{
// Serial.print(VOG); Serial.print("kts,  "); Serial.print(Course); Serial.println(" deg True");  // for debugging
   lcd.setCursor(0,3);                                     // cursor position for speed display
   if (disp_knots) {                                       // check is spped to be in knots or mph 
      lcd.print(VOG,1);                                    // display speed over ground (or water) in knots
      lcd.print(F("kts  "));
   }
   else {                                                  // show the speed in statute miles per hour
      lcd.print(VOG*1.15078,1);                            // convert NMEA's knots to mph and display
      lcd.print(F("mph"));
   }
   lcd.setCursor(10,3);                                  // cursor location for course heading
   if (VOG > 0.4) {                   // *** if  this line and the ones marked *** are uncommented, no course will display for very slow speeds
      if(Course < 10) lcd.print(F("00"));  // fix up leading zeros if needed
      else if(Course < 100) lcd.print(F("0"));
      lcd.print(Course,0);
      lcd.write(0xDF); //degree symbol
      lcd.print(F(" True "));
      }                             //   ***   needs to be uncommented if the above "if" statement is uncommented
   else {                           //   ***   needs to be uncommented if the above "if" statement is uncommented
      lcd.print(F(" - - - - -"));   //   ***   needs to be uncommented if the above "if" statement is uncommented
   }                                //   ***   needs to be uncommented if the above "if" statement is uncommented
}

// routine to prompt f0r input, then set thee RTC module MM:DD:YY day-of-week hh:mm with ss set to 00
void SetRTC()
{
   //  define some local variables
   unsigned long start = millis(); byte tst;
   byte tmpCnt = 6;        // number of clock parameters to set
   String tmpName[] = {    // array of names for selections
     "Mon ",    // month, MM  
     "Day ",    // Day of month, DD
     "Yr  ",    // Last two digits of year, YY
     "WkDy",    // Day of week, Sunday = 0
     "Hr  ",    // Hour, hh
     "Min ",    // minutes, mm
   };
  
   byte maxDat[] = {12,31,31,6,23,59};  // array of limiting values for setting parameters                                    
   lcd.clear(); lcd.setCursor(0,0);
   lcd.print(F("RTC Set:"));  delay(100);

   // now wait for any button press to start the RTC setting process
   // if no button press within 10 seconds, skip the RTC setting and go on to the main menu and normal operations
   tst=0; 
   while (tst == 0)
   {
      delay(100); Serial.println(" waiting to enter RTCset routine");
      if ((millis() - start) > 10000) return;  //  jump all the way out if 10 seconds elapse with no button press
      tst=TestBTNS();                          //  check buttons again  
   }

   // there was a button press within 10 sec, start the RTC setting process
   // it **must** go to all the way to completion, with all data items entered, there is no timeout

   rtc.getDateTime();  // get the current time from the RTC module and pre-load the setting parameters
   byte tmpDat[] = {rtc.getMonth(),rtc.getDay(),rtc.getYear(),rtc.getWeekday(),rtc.getHour(),rtc.getMinute()};  //MM, DD, YY, DofW, hh, mm   --  initialize all data
 
   for (byte i=0; i<=tmpCnt-1; i++)  // loop through all six parameters
   {
       lcd.setCursor(0,2); lcd.print(F("                    "));  // clear the bottom line
       tst=0;
       lcd.setCursor(0,2);                            // position for text on next-to-last line
       lcd.print(tmpName[i]);                         // write the name of the parameter to LCD
 
       tst=TestBTNS();                                // 1=shortpress MENU, 10=longpress MENU, 2=shortpress SEL, 20=longpress SEL
//     Serial.println(tst);  // for debugging
         
        // entering a numeric value
        byte tst2 = TestBTNS();
        int tset = tmpDat[i];  // fetch the current value, and make an integer to allow for negative numbers
        while (tst2 != 1)       // another short press of MENU button will exit this part
        {
            lcd.setCursor(9,2); lcd.print(tset); lcd.print("   ");  // clear remainder of data field
             if (tst2 == 2)                                          // look for a short press of the SEL button
             {
                tset = tset + 1;                  // increment the value of this parameter
                if (tset > maxDat[i]) tset = 0;  //  set a limit of 60, and roll over back to zero               
             }
             tst2 = TestBTNS();  // single short press of MENU exist this data entry part, another short press goes to next item             
         }  // end of inner "while" structure
             
         tmpDat[i] = tset;   // update the setting, which will get used later to set the RTC              
         delay(5);

   }  // end of 'for' loop

   // now send those settings to the RTC module
   // in order of Day,Weekday,Month,Century=00,Year,Hour,Minute,Second=0    .. the zeroes means century 2000, and 0 seconds
   rtc.setDateTime(tmpDat[1],tmpDat[3],tmpDat[0],0,tmpDat[2],tmpDat[4],tmpDat[5],0);   // defaults to year 20xx and 00 seconds

   lcd.setCursor(0,3); lcd.print("CLOCK SET"); delay(2000);
}  // end of RTC setting routine


